C-**********************************************************************
C-  The programs in this file are a part of                            *
C-      SIGNAL PROCESSING ALGORITHMS IN FORTRAN AND C                  *
C-      by S.D. Stearns and R.A. David                                 *
C-  Copyright 1993 by Prentice-Hall, Inc.                              *
C-                                                                     *
C-  Contact S.D. Stearns at MS 1159, Sandia National Labs.,            *
C-  Albuquerque, NM 87185                                              *
C-  phone 505-845-7382 -- email stearns@sandia.gov                     *
C-**********************************************************************
C-*                       FFTLIB.FOR                                   *
C-*            SUBPROGRAMS IN ALPHABETICAL ORDER                       *
C-*                      IN FORTRAN-77                                 *
C-*          (FOURIER TRANSFORM ROUTINES IN SPALIB)                    *
C-**********************************************************************
C-
      SUBROUTINE SPCHRP(X,LX,LX1,F1,F2,WORK,LX2,IERROR)
C-LATEST DATE: 11/27/85
C-COMPUTES THE CHIRP-Z TRANSFORM OF A COMPLEX SEQUENCE.
C-X=COMPLEX INPUT ARRAY CONTAINING THE COMPLEX SEQUENCE.
C-LX=LAST INDEX IN THE ARRAY X(0:LX).  LX+1 MUST BE A POWER OF 2.
C-LX1=LAST INDEX OF THE COMPLEX INPUT SEQUENCE, X(0)...X(LX1).
C-    LX1 MUST BE LESS THAN LX.
C-F1=FREQUENCY OF FIRST DFT COMPONENT.  (SAMPLING FREQ.=1.0.)
C-F2=FREQUENCY OF LAST DFT COMPONENT, GREATER THEN F1.
C-WORK=COMPLEX WORK ARRAY, DIMENSIONED COMPLEX WORK(0:LX).
C-AFTER EXECUTION DFT COMPONENTS, SPACED EVENLY FROM F1 THRU F2,
C-  ARE STORED IN X(0)...X(LX2).  THE ORIGINAL TIME SERIES IS LOST.
C-LX2=LAST FREQUENCY INDEX AS ABOVE, COMPUTED AS LX2=LX-LX1 DURING
C-     EXECUTION OF THE ROUTINE.
C-IERROR=0  NO ERRORS
C-       1  LX IS NOT GREATER THAN LX1
C-       2  F2 IS NOT GREATER THAN F1
C-       3  LX+1 IS NOT A POWER OF 2
      COMPLEX X(0:LX),WORK(0:LX),AM1,W2
      TP=8.*ATAN(1.)
      IERROR=1
      LX2=LX-LX1
      IF(LX2.LT.1) RETURN
      IERROR=2
      IF(F1.GE.F2) RETURN
      IERROR=3
      LXC=1
    1 LXC=2*LXC+1
      IF(LXC-LX) 1,2,6
    2 AM1=EXP(CMPLX(0.,-TP*F1))
      W2=EXP(CMPLX(0.,-TP*(F2-F1)/(2.*LX2)))
      DO 3 K=0,LX
        IF(K.LE.LX1) WORK(K)=(AM1*W2**K)**K*X(K)
        IF(K.GT.LX1) WORK(K)=0.
        IF(K.LE.LX2) X(K)=W2**(-K*K)
        IF(K.GT.LX2) X(K)=W2**(-(LX+1-K)**2)
    3 CONTINUE
      CALL SPFFTC(WORK,LX+1,-1)
      CALL SPFFTC(X,LX+1,-1)
      DO 4 K=0,LX
        X(K)=X(K)*WORK(K)
    4 CONTINUE
      CALL SPFFTC(X,LX+1,+1)
      DO 5 K=0,LX2
        X(K)=X(K)*W2**(K*K)/(LX+1)
    5 CONTINUE
      IERROR=0
    6 RETURN
      END
C-
C-**********************************************************************
C-
      SUBROUTINE SPDFTC(X,Y,N,ISIGN)
C-LATEST DATE: 02/20/87
C-COMPUTES THE DFT OF A COMPLEX VECTOR X HAVING N COMPLEX SAMPLES.
C-X AND Y ARE SPECIFIED COMPLEX X(0:N-1),Y(0:N-1) OR LARGER.
C-INPUTS ARE X(0),X(1),...,X(N-1)=COMPLEX DATA VECTOR.
C-           N=NUMBER OF DATA SAMPLES.
C-           ISIGN=-1 FOR FORWARD OR +1 FOR INVERSE TRANSFORM.
C-OUTPUTS ARE Y(0),Y(1),...,Y(N-1)=COMPLEX DFT COMPONENTS, WITH
C-  COSINE COMPONENT=REAL PART AND SINE COMPONENT=IMAGINARY PART.
C-Y AND X CANNOT BE THE SAME ARRAY.
      COMPLEX X(0:N-1),Y(0:N-1),TPJN
      TPJN=CMPLX(0.,ISIGN*8.*ATAN(1.)/N)
      DO 2 M=0,N-1
        Y(M)=X(0)
        DO 1 K=1,N-1
          Y(M)=Y(M)+X(K)*EXP(TPJN*K*M)
    1   CONTINUE
    2 CONTINUE
      RETURN
      END
C-
C-**********************************************************************
C-
      SUBROUTINE SPDFTR(X,Y,N)
C-LATEST DATE: 10/26/90
C-COMPUTES THE DFT OF A REAL VECTOR X CONTAINING N DATA SAMPLES.
C-X IS DIMENSIONED X(0:N-1) OR LARGER.
C-Y IS COMPLEX Y(0:N/2) IF N EVEN, OR Y(0:(N-1)/2) IF N ODD.
C-INPUTS ARE X(0),X(1),...,X(N-1) AND N=NUMBER OF DATA SAMPLES.
C-OUTPUTS Y(0),Y(1),...,Y(N/2) IF N EVEN, OR Y(0),Y(1),...,
C-  Y((N-1)/2) IF N ODD, ARE THE COMPLEX DFT COMPONENTS, WITH
C-  COSINE COMPONENT=REAL PART AND SINE COMPONENT=IMAGINARY PART.
C-Y AND X CANNOT BE THE SAME ARRAY.
      DIMENSION X(0:N-1)
      COMPLEX Y(0:N/2),TPJN
      TPJN=CMPLX(0.,-8.*ATAN(1.)/N)
      DO 2 M=0,N/2
        Y(M)=X(0)
        DO 1 K=1,N-1
          Y(M)=Y(M)+X(K)*EXP(TPJN*K*M)
    1   CONTINUE
    2 CONTINUE
      RETURN
      END
C-
C-**********************************************************************
C-
      SUBROUTINE SPFFTC(X,N,ISIGN)
C-LATEST DATE: 02/20/87
C-FAST FOURIER TRANSFORM OF N=2**K COMPLEX DATA POINTS USING TIME
C-DECOMPOSITION WITH INPUT BIT REVERSAL.  N MUST BE A POWER OF 2.
C-X MUST BE SPECIFIED COMPLEX X(0:N-1) OR LARGER.
C-INPUT IS N COMPLEX SAMPLES, X(0),X(1),...,X(N-1).
C-COMPUTATION IS IN PLACE, OUTPUT REPLACES INPUT.
C-ISIGN = -1 FOR FORWARD TRANSFORM, +1 FOR INVERSE.
C-X(0) BECOMES THE ZERO TRANSFORM COMPONENT, X(1) THE FIRST,
C-AND SO FORTH.  X(N-1) BECOMES THE LAST COMPONENT.
      COMPLEX X(0:N-1),T
      PISIGN=4*ISIGN*ATAN(1.)
      MR=0
      DO 2 M=1,N-1
        L=N
    1   L=L/2
        IF(MR+L.GE.N) GO TO 1
        MR=MOD(MR,L)+L
        IF(MR.LE.M) GO TO 2
        T=X(M)
        X(M)=X(MR)
        X(MR)=T
    2 CONTINUE
      L=1
    3 IF(L.GE.N) RETURN
      DO 5 M=0,L-1
        DO 4 I=M,N-1,2*L
          T=X(I+L)*EXP(CMPLX(0.,M*PISIGN/FLOAT(L)))
          X(I+L)=X(I)-T
          X(I)=X(I)+T
    4   CONTINUE
    5 CONTINUE
      L=2*L
      GO TO 3
      END
C-
C-**********************************************************************
C-
      SUBROUTINE SPFFTR(X,N)
C-LATEST DATE: 10/26/90
C-FFT ROUTINE FOR REAL TIME SERIES (X) WITH N=2**K SAMPLES.
C-COMPUTATION IS IN PLACE, OUTPUT REPLACES INPUT.
C-INPUT:  REAL VECTOR X(0:N+1) WITH REAL DATA SEQUENCE IN FIRST N
C-        ELEMENTS; ANYTHING IN LAST 2.  NOTE: X MAY BE DECLARED
C-        REAL IN MAIN PROGRAM PROVIDED THIS ROUTINE IS COMPILED 
C-        SEPARATELY ... COMPLEX OUTPUT REPLACES REAL INPUT HERE.
C-OUTPUT: COMPLEX VECTOR XX(O:N/2), SUCH THAT X(0)=REAL(XX(0)),X(1)=
C-        IMAG(XX(0)), X(2)=REAL(XX(1)), ..., X(N+1)=IMAG(XX(N/2).
C-IMPORTANT:  N MUST BE AT LEAST 4 AND MUST BE A POWER OF 2.
      COMPLEX X(0:N/2),U,TMP
      TPN=8.*ATAN(1.)/N
      CALL SPFFTC(X,N/2,-1)
      X(N/2)=X(0)
      DO 1 M=0,N/4
        U=CMPLX(SIN(M*TPN),COS(M*TPN))
        TMP=((1.+U)*X(M)+(1.-U)*CONJG(X(N/2-M)))/2.
        X(M)=((1.-U)*X(M)+(1.+U)*CONJG(X(N/2-M)))/2.
        X(N/2-M)=CONJG(TMP)
    1 CONTINUE
      RETURN
      END
C-
C-**********************************************************************
C-
      SUBROUTINE SPIDTR(Y,X,N)
C-LATEST DATE: 11/12/85
C-SAME AS SPDFTR EXCEPT REVERSED; Y=SPECTRUM, X=OUTPUT TIME SERIES.
C-SPECTRAL DATA IS ASSUMED TO BE IN COMPLEX Y(0) THRU Y(N/2).
C-N=TIME SERIES LENGTH SHOULD BE EVEN.  TIME SERIES (SCALED BY N)
C-IS COMPUTED IN X(0) THRU X(N-1).  COMPLEX Y(0:N/2) AND DIMENSION
C-X(0:N-1) ARE ASSUMED, ALTHOUGH THE ARRAYS MAY BE LARGER.
      DIMENSION X(0:N-1)
      COMPLEX Y(0:N/2),TPJN
      TPJN=CMPLX(0.,8.*ATAN(1.)/N)
      DO 2 K=0,N-1
        X(K)=Y(0)+Y(N/2)*(-1)**K
        DO 1 M=1,N/2-1
          X(K)=X(K)+2.*REAL(Y(M)*EXP(TPJN*K*M))
    1   CONTINUE
    2 CONTINUE
      RETURN
      END
C-
C-**********************************************************************
C-
      SUBROUTINE SPIFTR(X,N)
C-LATEST DATE: 02/20/87
C-INVERSE FFT OF THE COMPLEX SPECTRUM OF A REAL TIME SERIES.
C-X AND N ARE THE SAME AS IN SPFFTR.  IMPORTANT: N MUST BE A POWER
C-OF 2 AND X MUST BE DIMENSIONED X(0:N+1) (REAL ARRAY, NOT COMPLEX).
C-THIS ROUTINE TRANSFORMS THE OUTPUT OF SPFFTR BACK INTO THE INPUT,
C-SCALED BY N.  COMPUTATION IS IN PLACE, AS IN SPFFTR.
      COMPLEX X(0:N/2),U1,TMP
      TPN=8.*ATAN(1.)/N
      DO 1 M=0,N/4
        U1=CMPLX(SIN(M*TPN),-COS(M*TPN))
        TMP=(1.+U1)*X(M)+(1.-U1)*CONJG(X(N/2-M))
        X(M)=(1.-U1)*X(M)+(1.+U1)*CONJG(X(N/2-M))
        X(N/2-M)=CONJG(TMP)
    1 CONTINUE
      CALL SPFFTC(X,N/2,1)
      RETURN
      END
C-
C-**********************************************************************
C-
C-**********************************************************************
C-
C-DOUBLE-PRECISION VERSIONS OF ROUTINES IN FFTLIB.
C-THESE WERE ORIGINALLY IN DPLIB.  APPENDED TO SPALIB 01/08/93.
C-SAM D. STEARNS, SANDIA NATIONAL LABS DEPT. 9311.  505-845-7382.
C-
C-**********************************************************************
C-
      SUBROUTINE DPFFT(X,N)
c-Double precision version of SPFFT -- Sept. 11, 1991.
c-Latest date: 12/04/87
c-FFT using time decomposition with input bit reversal.
c-X(0:N+1)=REAL*8 array holding input sequence X(0),---,X(N-1), plus
c-         2 extra elements X(N),X(N+1), which need not be initialized.
c-Number of samples (N) must be a power of 2 greater than 2.
c-After execution, 1st FFT component is X(0)=real, X(1)=imaginary,
c-                 2nd FFT component is X(2)=real, X(3)=imaginary,
c-                                    etc.,
c-          N/2 (last) FFT component is X(N)=real, X(N+1)=imaginary.
c-
      real*8 pi
      complex*16 X(0:N/2),T,U
      PI=4.*DATAN(1.d0)
      MR=0
      DO 2 M=1,N/2-1
       L=N/2
    1  L=L/2
        IF(MR+L.GE.N/2) GO TO 1
       MR=MOD(MR,L)+L
       IF(MR.LE.M) GO TO 2
       T=X(M)
       X(M)=X(MR)
       X(MR)=T
    2 CONTINUE
      L=1
    3 IF(L.LT.N/2) THEN
       DO 5 M=0,L-1
        DO 4 I=M,N/2-1,2*L
         T=X(I+L)*CdEXP(DCMPLX(0.d0,-M*PI/DFLOAT(L)))
         X(I+L)=X(I)-T
         X(I)=X(I)+T
    4   CONTINUE
    5  CONTINUE
       L=2*L
       GO TO 3
      ENDIF
    6 X(N/2)=X(0)
      DO 7 M=0,N/4
       U=dCMPLX(dSIN(M*2.*PI/N),dCOS(M*2.*PI/N))
       T=((1.+U)*X(M)+(1.-U)*dCONJG(X(N/2-M)))/2.
       X(M)=((1.-U)*X(M)+(1.+U)*dCONJG(X(N/2-M)))/2.
       X(N/2-M)=dCONJG(T)
    7 CONTINUE
      RETURN
      END
C-
C-**********************************************************************
C-
      SUBROUTINE DPFFTC(X,N,ISIGN)
C-LATEST DATE: 09/27/91
C-D.P. FAST FOURIER TRANSFORM OF N=2**K DBLE. COMPLEX DATA POINTS USING
C-TIME DECOMPOSITION WITH INPUT BIT REVERSAL.  N MUST BE A POWER OF 2.
C-X MUST BE SPECIFIED DBLE. COMPLEX X(0:N-1) OR LARGER.
C-INPUT IS N DBLE. COMPLEX SAMPLES, X(0),X(1),...,X(N-1).
C-COMPUTATION IS IN PLACE, OUTPUT REPLACES INPUT.
C-ISIGN = -1 FOR FORWARD TRANSFORM, +1 FOR INVERSE.
C-X(0) BECOMES THE ZERO TRANSFORM COMPONENT, X(1) THE FIRST,
C-AND SO FORTH.  X(N-1) BECOMES THE LAST COMPONENT.
      complex*16 X(0:N-1),T
      REAL*8 PISIGN
      PISIGN=4.D0*ISIGN*DATAN(1.D0)
      MR=0
      DO 2 M=1,N-1
        L=N
    1   L=L/2
        IF(MR+L.GE.N) GO TO 1
        MR=MOD(MR,L)+L
        IF(MR.LE.M) GO TO 2
        T=X(M)
        X(M)=X(MR)
        X(MR)=T
    2 CONTINUE
      L=1
    3 IF(L.GE.N) RETURN
      DO 5 M=0,L-1
        DO 4 I=M,N-1,2*L
          T=X(I+L)*CDEXP(DCMPLX(0.D0,M*PISIGN/DFLOAT(L)))
          X(I+L)=X(I)-T
          X(I)=X(I)+T
    4   CONTINUE
    5 CONTINUE
      L=2*L
      GO TO 3
      END
C-
C-**********************************************************************
C-
      SUBROUTINE DPIFFT(X,N)
c-Double precision version of SPIFFT -- September 16, 1991.
c-Latest date: 08/24/88
c-Inverse FFT. Transforms SPFFT output back into N times original input.
c-X(0:N+1) is a REAL array holding the complex DFT components with
c-         indices from 0 through N/2 in order as described in SPFFT.
c-As in SPFFT, N must be a power of 2 greater than 2.
c-
      real*8 pi
      complex*16 X(0:N/2),T,U
      PI=4.d0*dATAN(1.d0)
      DO 1 M=0,N/4
       U=CMPLX(dSIN(M*2.*PI/N),-dCOS(M*2.*PI/N))
       T=(1.+U)*X(M)+(1.-U)*dCONJG(X(N/2-M))
       X(M)=(1.-U)*X(M)+(1.+U)*dCONJG(X(N/2-M))
       X(N/2-M)=dCONJG(T)
    1 CONTINUE
      MR=0
      DO 3 M=1,N/2-1
       L=N/2
    2  L=L/2
        IF(MR+L.GE.N/2) GO TO 2
       MR=MOD(MR,L)+L
       IF(MR.LE.M) GO TO 3
       T=X(M)
       X(M)=X(MR)
       X(MR)=T
    3 CONTINUE
      L=1
    4 IF(L.LT.N/2) THEN
       DO 6 M=0,L-1
        DO 5 I=M,N/2-1,2*L
         T=X(I+L)*cdEXP(dCMPLX(0.d0,M*PI/FLOAT(L)))
         X(I+L)=X(I)-T
         X(I)=X(I)+T
    5   CONTINUE
    6  CONTINUE
       L=2*L
       GO TO 4
      ENDIF
      RETURN
      END
C-
C-**********************************************************************
