function [d,c]=spbwcf(L,k)
% [d,c]=spbwcf(L,k)
% Generates kth section coefficients for Lth order
% normalized lowpass Butterworth filter.
% Described in Section 7.3. See also Eqs. 7.4, 7.14.
%
% Inputs:
%   L = Order (# poles) of normalized low-pass analog filter.
%   k = Section number.
%       If L is even, there are L/2 2-pole sections.
%       If L is odd, there are (L-1)/2 2-pole sections plus
%        one additional 1-pole section.
%
% Outputs:  
%    d = D(s) coeff. vector for kth section. See Eq. 7.4.
%    c = C(s) coeff. vector for kth section. See Eq. 7.4.

if L<=0
   error('SPBWCF: Filter order (L) must be >= 0.');
end
if (k<=0)|(k>fix((L+1)/2))
   error('SPBWCF: Filter section number is too large or small.');
end
d=1; c=1;
tmp=k-(L+1)/2;
if tmp==0
   c(2)=1;
else
   c(2)=-2*cos((2*k+L-1)*pi/(2*L));
   c(3)=1;
end
return
