function [d,c]=spchbi(L,k,ep)
% [d,c]=spchbi(L,k,ep)
% Generates kth section coefficients for Lth order normalized
% lowpass Chebyshev type I analog filter.  ep=passband ripple
% constant.  See Section 7.4 and Eq. 7.24.
% Inputs:
%    L  = Order of normalized low-pass analog filter.
%         Odd order L:   Last section contains a single pole.
%         Even order L:  Trans. func. scaled in first section.
%    k  = Section number. For 2nd order sections:  k<=(L+1)/2.
%    ep = Epsilon in Eq. 7.24; regulates the passband ripple.
% Outputs:  
%    d      = D(s) coefficient vector.
%    c      = C(s) coefficient vector.

if (L<=0) | (k<=0) | (k>fix(L+1)/2),
   error('SPCHBI: called with invalid (L,k) combination.');
end
gam=((1+sqrt(1+ep^2))/ep)^(1/L);
sigma=0.5*(-gam+1/gam)*sin((2*k-1)*pi/(2*L));
omega=0.5*(+gam+1/gam)*cos((2*k-1)*pi/(2*L));
if (fix(L/2)~=fix((L+1)/2))&(k==(L+1)/2),
   d(1)=-sigma;
   c(1)=d(1);
   c(2)=1;
else
   c(1)=sigma^2+omega^2;
   c(2)=-2*sigma;
   c(3)=1;
   d(1)=c(1);
   if (fix(L/2)==fix((L+1)/2))&(k==1),
      d(1)=d(1)/sqrt(1+ep^2);
   end
end
return
