function X=spchrp(x,f1,f2,M)
% X=spchrp(x,f1,f2,M)
% X(1:M)= chirp-z transform row vector x.  f1,f2= limits
% frequency axis in Hz-s; M= number of components in X.
% Described in Chapter 14, Section 14.5.
% Inputs:
%    x   = row vector, real or complex.
%    f1  = frequency in Hz-s of first DFT component.
%    f2  = frequency in Hz-s of last DFT component; 0.5>f2>f1.
%    M   = number of transform components, >=2.
% Output:  
%    X(1:M) = chirp-z transform of x. The M DFT components are
%             spaced evenly from f1 through f2.

if(f1<0) | (f2<=f1) | (0.5<f2),
   error('SPCHRP: Frequency f1 and/or f2 out of range.');
end
if(M<2),
   error('SPCHRP: M is < 2.');
end
[nr,nc]=size(x);
if(nr~=1),
   error('SPCHRP: Data vector "x" is not a row vector.');
end
NM=length(x)+M-1;
w0=2*pi*f1; dw=2*pi*(f2-f1)/(M-1);
A=exp(-j*w0);
W=exp(-j*dw/2);
k=[0:NM-1];
xk=[x,zeros(1,M-1)];
yk=xk .* ((A .* W.^[0:NM-1]) .^ [0:NM-1]);
Wk2=[W.^(-([0:M-1]).^2),W.^(-(NM-[M:NM-1]).^2)];
gm=ifft(fft(yk) .* fft(Wk2));
X=gm(1:M) .* W.^((0:M-1).^2);
return
