function [cxy,M]=spconv(x,y)
% [cxy,M]=spconv(x,y)
% Fast convolution of row vectors x and y.
% Output is cxy in Equation 9.5.
% M = length of cxy. Uses the MATLAB fft and ifft functions.

% Begin: Determine the nonzero lengths.
Ny=length(y); Nx=length(x); M=Nx+Ny-1;
for Ly=Ny:-1:1,
   if y(Ly)~=0,
      break
   end
end
for Lx=Nx:-1:1,
   if x(Lx)~=0,
      break
   end
end
% Lengthen y and x if need be.  L=transform length.
L=Nx;
if(Ly>L-Lx),
   x=[x,zeros(1,Ly-(L-Lx))];
   L=L+Ly-(L-Lx);
end
if(L>Ny),
   y=[y,zeros(1,L-Ny)];
end
% Do the convolution and set length of cxy.
X=fft(x,L);
Y=fft(y,L);
cxy=real(ifft(X .* Y,L));
if(length(cxy)<M),
   cxy=[cxy,zeros(1,M-length(cxy))];
end
cxy=cxy(1:M);
% Reset lengths of x and y.
y=y(1:Ny); x=x(1:Nx);
return
