function [rxy,M]=spcorr(x,y)
% [rxy,M]=spcorr(x,y)
% Fast correlation of row vectors x and y.
% Output is rxy in Equation 9.6.
% M = length of rxy. Uses the MATLAB fft and ifft functions.

% Begin: Determine the nonzero lengths.
Ny=length(y); Nx=length(x); M=Ny;
for Fy=1:Ny,
   if y(Fy)~=0,
      break
   end
end
for Lx=Nx:-1:1,
   if x(Lx)~=0,
      break
   end
end
% Lengthen y and x if need be.  L=Transform size.
L=Nx;
if(M-(Fy-1) > L-Lx),
   x=[x,zeros(1,M-(Fy-1)-(L-Lx))];
   L=L+M-(Fy-1)-(L-Lx);
end
if(L>Ny),
   y=[y,zeros(1,L-Ny)];
end
% Do the correlation and set the length of rxy.
X=fft(x,L);
Y=fft(y,L);
rxy=real(ifft(conj(X) .* Y,L));
if(length(rxy)<M),
   rxy=[rxy,zeros(1,M-length(rxy))];
end
% Set the length of rxy.
rxy=rxy(1:M);
% Reset the lengths of x and y.
x=x(1:Nx); y=y(1:Ny);
return
