function [y,Nd]=spdeci(x,N,ratio)
% [y,Nd]=spdeci(x,N,ratio)
% Linearly decimates the sequence x(1:N).
% ratio=output/input T2/T1 ratio. See equation (10.1).
% Inputs: x, N, and ratio.
% Outputs: y(1:Nd) = decimated sequence; Nd =~ N/ratio.

if (ratio<1 | (ratio>N) | N<2),
   error('SPDECI: ratio<1 or ratio>N or N<2');
end
Nd=fix((N-1)/ratio)+1;
y(1:Nd)=x(1:Nd);
for k=1:Nd-1
  k1=fix(k*ratio);
  y(k+1)=x(k1+1)+(k*ratio-k1)*(x(min(N,k1+2))-x(k1+1));
end
return
