function h=spganc(b,a,freq)
% h=spganc(b,a,freq)
% See Chapter 7, Exercise 7.20.
% Cascade version of spgain. b & a are matrices dimensioned
% Ns x N and Ns x M, respectively, where Ns=number of filter
% sections and N-1,M-1=orders of b,a in all sections.
% Transfer function is composed of Ns sections in cascade
% with mth stage transfer function defined by:
%
%                  b(m,1)+b(m,2)*z^(-1)+...+b(m,N)*z^(-(N-1))
%           H(z) = ------------------------------------------
%                  a(m,1)+a(m,2)*z^(-1)+...+a(m,M)*z^(-(M-1))
%
% Inputs:
%    b      = B(z) coefficients. 1 section per row. Ns x N.
%    a      = A(z) coefficients. 1 section per row. Ns x M.
%
%    freq   = Frequency in Hz-sec; i.e., sampling frequency = 1.0.
%             Note: freq can be a scalar or a row vector.
% Outputs:  
%    h      = Complex gain of H(Z) at frequency = freq Hz-s.

[Ns,N]=size(b); [Nsa,M]=size(a);
if Ns~=Nsa,
   error('SPGANC: a and b do not have same # of rows.');
end

h=ones(1,length(freq));
for ns=1:Ns,
   h=h .* spgain(b(ns,:),a(ns,:),freq);
end;
return
