function [y,pv]=splflt(kappa,nu,x,pv)
% [y,pv]=splflt(kappa,nu,x,pv)
% Filters row vector x using lattice filter with coefficients in
% row vectors kappa and nu. pv=internal values, updated to enable
% block mode filtering if needed.  Initially, pv=0. See Sect. 6.7.
%
% Inputs:
%    kappa = Feedback coefficients vector, 1x(M-1).
%    nu    = Feedforward coefficients vector, 1xM.
%    x     = Input signal row vector.
%    pv    = Vector with past values.  pv=0 for zero initial cond.
% Outputs:  
%    y     = In-place vector with filter outputs.
%    pv    = Updated past value vector; 1xK.  May be scalar zero.
%
% See also: spltcf.

Nx=length(x); Nk=length(kappa); M=length(nu); Np=length(pv);
if(M ~= Nk+1),
   error('SPLFLT: length(nu) must equal (length(kappa)+1).');
end
if(Np < M),
   pv=[pv,zeros(1,M-Np)];
end
for k=0:Nx-1,
   ysum=x(k+1);
   for n=M-1:-1:1,
      ysum=ysum-kappa(n)*pv(n);
      pv(n+1)=pv(n)+kappa(n)*ysum;
   end
   pv(1)=ysum;
   y(k+1)=sum(nu.*pv(1:M));
   if abs(x(k+1))>1e10,
      error('SPLFLT: Output has exceeded 1.e10.');  
   end
end
pv=pv(1:M);
return
