function P=splsmt(N,L)
% P=splsmt(N,L)
% Generates matrix for orthogonal polynomials.
% N=no. data points; L=order of polynomial in
% eq. 11.13; P=matrix in eq. 11.18, (L+1)x(L+1).
% Inputs:
%    N = number of data samples; scalar.
%    L = order of f(x) in eq. 11.13.
% Outputs:  
%    P = coeff. matrix in eq. 11.18

P=tril(ones(L+1));
for i=1:L,
   for j=1:i,
     P(i+1,j+1)=((-1)^j)*spbfct(i,j)*spbfct(i+j,j)/...
                 (spbfct(N-1,j)*(spbfct(j,j)^2));
   end
end
return
