function [y,tsv]=spmask(x,itype)
% [y,tsv]=spmask(x,itype)
% Applies a data window to (masks) any row vector, x.
% Inputs:  x=data (row vector); itype=window type (1-6).
% Outputs: y=masked version of x; tsv=sum of squared window values.
%
% itype  = 1(Rectangular); 2(Tapered rectangular); 3(Triangular);
%          4(Hanning);     5(Hamming);             6(Blackman).
% tsv    = Energy of window vector = sum of squared window values.

if (itype<1)|(itype>6)
   error('SPMASK: itype (2ND argument) out of range.');
end
N=length(x); k=[1:N];
mask=spwndo(itype,N,k);
tsv=sum(mask .^ 2);
y=mask .* x;
end
return
