function [orthb,P]=sporth(y,L)
% [orthb,P]=sporth(y,L)
% Coefficients for l.s.fit via orthogonal polynomials.
% y=data vector; L=order in eq. 11.13; orthb=b vector
% in eq. 11.13 (column of length L+1).
% Inputs:
%  y   = row vector of equally spaced data samples;
%  L   = Order of system of equations; scalar.
% Outputs:
%  orthb = Orthogonal polynomial coeff. col. vector.
%  P     = (L+1)x(L+1) matrix in equation 11.18.

N=length(y);
if (N<L)|(L<=0),
   error('SPORTH: L or no. data points too small.');
end
P=splsmt(N,L);
orthb=zeros(L+1,1);
for i=0:L,
   sn=0; sd=0;
   for nn=0:N-1,
      sum=0;
      for j=0:min(i,nn),
         sum=sum+P(i+1,j+1)*spbfct(nn,j);
      end
      sn=sn+y(nn+1)*sum;
      sd=sd+sum^2;
   end
   orthb(i+1)=sn/sd;
end
return
