function [pds,nsgmts]=sppowr(x,NDFT,iwindo,ovrlap)
% [pds,nsgmts]=sppowr(x,NDFT,iwindo,ovrlap)
% Power density spectrum, averaged over segments of x.
% NDFT=DFT size=twice spectrum size, iwindo=window
% type, ovrlap=overlap. Described in Chapter 4 of SPA.
% Inputs:
%    x      = Input row vector, x(1:N).
%    NDFT   = DFT segment length.  # spectral components=NDFT/2+1.
%    iwindo = Data window type:
%                1) Rectangular.                 4) Hanning.
%                2) Tapered rectangular.         5) Hamming.
%                3) Triangular.                  6) Blackman.
%    ovrlap = Fraction that each data segment of size NDFT
%                overlaps its predecessor.  Must be greater than
%                or equal 0 and less than 1.
% Outputs
%    pds    = Output power density spectrum; length=NDFT/2+1.
%    nsgmts = Number of overlapping segments of x averaged together.

if(size(x,1)~=1),
   x=x';
end

[Nrows,N]=size(x); M=fix(NDFT/2)+1;
if(Nrows~=1),
   error('SPPOWR: x must be a vector.');
elseif(NDFT<8),
   error('SPPOWR: DFT size is < 8.');
elseif N<NDFT,
   error('SPPOWR: Length of x is < DFT size.');
end

pds(1:M)=zeros(1,M);
nshift=fix(min(NDFT,max(1,fix(NDFT*(1-ovrlap)+.5))));
nsgmts=fix(1+(N-NDFT)/nshift);
for isegmt=0:nsgmts-1
   temp=x((nshift*isegmt+1):(nshift*isegmt+NDFT));
   [temp,tsv]=spmask(temp(1:NDFT),iwindo);
   TEMP=fft(temp,NDFT);
   pds=pds+TEMP(1:M).*conj(TEMP(1:M))/(tsv*nsgmts);
end
return
