function y=spresp(x,Ny,b,a)
% y=spresp(x,Ny,b,a)
% y(1:Ny)=time domain response of a specified causal linear system.
% x=vector of input samples assumed to continue with final value.
% b,a=weight vectors of linear system.  See Chapter 5, Section 5.7.
%
%                      b(1)+b(2)*Z^(-1)+...+b(N)*Z^(-(N-1))
%               H(Z) = ------------------------------------
%                      a(1)+a(2)*Z^(-1)+...+a(M)*Z^(-(M-1))
%
%           For an FIR system, use M=1 and a(1)=1.
%
%           Zero initial conditions are assumed.
%
% Inputs:
%  x  = user-supplied input signal with first sample at x(1). This
%       signal is assumed to continue with its final value. Thus,
%       a unit impulse is [1,0], a unit step could be 1, etc.
%  Ny = length of output y(1:Ny).
%  b  = B(z) coefficient vector.
%  a  = A(z) coefficient vector.
%
% Outputs:  
%  y(1:Ny) = Computed response of H(z)=B(z)/A(z) to input signal x.

if(a(1)==0),
   error('SPRESP: a(1) must not be zero.');
end
if (a(1)~=1)
   a=a/a(1);
   b=b/a(1);
end
for k=0:Ny-1,
   sum=0;
   for n=0:length(b)-1,
      sample=0;
      if (k-n)>=0,
         sample=x(min([k-n+1,length(x)]));
      end
      sum=sum+b(n+1)*sample;
   end
   for n=1:length(a)-1,
      sample=0;
      if (k-n)>=0,
         sample=y(k-n+1);
      end
      sum=sum-a(n+1)*sample;   
   end
   y(k+1)=sum;
end
return
