function y=spwlsh(logN,nseq,k)
% y=spwlsh(logN,nseq,k)
% Generates one or a set of Walsh-ordered Walsh coefficients.
% Inputs:
%    logN = log base 2 of transform array size; scalar.
%    nseq = sequence index (0 through 2^(logn-1);
%           must be offset by calling routine)); scalar.
%    k    = time index(es) (0 - 2^logN-1); scalar or row vector.
% Outputs:
%    y    = Walsh coefficient(s), either -1.0 or +1.0; 
%           row vector with length same as k.

if(max(k)>=2^logN | min(k)<0),
   error('SPWLSH: An element of k is outside allowable range.');
end
is1=2^(logN-1); is2=2;
y=rem(fix(nseq/is1),2) * rem(k,2);
for i=2:logN,
   y=y+(rem(fix(nseq/is1),2)+rem(fix(nseq/(is1/2)),2))*rem(fix(k/is2),2);
   is1=is1/2;
   is2=is2*2;
end
y=1-2*rem(y,2);
return
