function w=spwndo(itype,N,k)
% w=spwndo(itype,N,k)
% Computes w = a single sample of a data window.
% Inputs: itype=window type; N=window size; k=sample # (1-N).
% See Chapter 14, Section 14.2.
%    itype  = 1) Rectangular.                 4) Hanning.
%             2) Tapered rectangular.         5) Hamming.
%             3) Triangular.                  6) Blackman.
%    N      = Size of (total number of samples in) window.
%    k      = Sample number within window, from 1 through N;
%             (May also be a row vector with elements in [1,N].
% Outputs:  
%    w      = window value(s).

w=zeros(1,length(k));
if (itype<1)|(itype>6)|(min(k)<1)|(max(k)>N)
   error('SPWNDO: k must be in [1,N] and itype in [1,6].');
end
if itype==1
   w=ones(1,length(k));
elseif itype==2
   L=fix((N-2)/10);
   w=ones(1,length(k));
   for index=1:length(k),
      if k(index)-1<=L
         w(index)=0.5*(1-cos((k(index)-1)*pi/(L+1)));
      elseif k(index)-1>(N-L-2)
         w(index)=0.5*(1-cos((N-k(index))*pi/(L+1)));
      end
   end
elseif itype==3
   w=1-abs(1-2*(k-1)/(N-1));
elseif itype==4
   w=0.5*(1-cos(2*(k-1)*pi/(N-1)));
elseif itype==5
   w=0.54-0.46*cos(2*(k-1)*pi/(N-1));
else
   w=0.42-0.5*cos(2*(k-1)*pi/(N-1))+0.08*cos(4*(k-1)*pi/(N-1));   
end     
return
